import pytest

from ofxstatement_nordigen.schemas import NordigenTransactionModel


@pytest.mark.parametrize(
    "data",
    [
        {
            "transactionId": "123456789",
            "transactionAmount": {"amount": 100.0, "currency": "EUR"},
            "valueDate": "2023-10-01",
            "valueDateTime": "2023-10-01T12:00:00Z",
            "remittanceInformationStructured": "Payment for invoice #12345",
        },
        {
            "transactionId": "987654321",
            "entryReference": "REF123456",
            "bookingDate": "2025-03-31",
            "bookingDateTime": "2025-03-31T00:00:00+00:00",
            "transactionAmount": {"amount": "-1521.00", "currency": "GBP"},
            "remittanceInformationUnstructured": "Payment for invoice #67890",
            "additionalInformation": "Payment received",
            "proprietaryBankTransactionCode": "BP",
            "internalTransactionId": "INT123456",
        },
        {
            "transactionId": "anonymized_transaction_id",
            "entryReference": "anonymized_entry_reference",
            "bookingDate": "2025-04-05",
            "valueDate": "2025-04-05",
            "bookingDateTime": "2025-04-05T00:00:00+00:00",
            "valueDateTime": "2025-04-05T00:00:00+00:00",
            "transactionAmount": {"amount": "0.00", "currency": "XXX"},
            "currencyExchange": {"sourceCurrency": "XXX", "exchangeRate": "0.0"},
            "remittanceInformationUnstructured": "anonymized_remittance_information",
            "additionalInformation": "anonymized_additional_information",
            "additionalDataStructured": {
                "CardSchemeName": "anonymized_card_scheme",
                "Name": "anonymized_name",
                "Identification": "anonymized_identification",
            },
            "internalTransactionId": "anonymized_internal_transaction_id",
        },
    ],
)
def test_go_cardless_transaction_model(data):
    validated = NordigenTransactionModel(**data)
    assert validated is not None



from __future__ import annotations

import datetime
from ofxstatement.statement import Currency
from decimal import Decimal
from typing import List, Optional, Union

from pydantic import BaseModel, field_validator


class Amount(BaseModel):
    amount: Decimal
    currency: Currency

    class Config:
        arbitrary_types_allowed = True

    @field_validator("currency", mode="before")
    def validate_currency(cls, value):
        if isinstance(value, str):
            return Currency(value)
        return value


class InstructedAmount(Amount):
    targetCurrency: Optional[Currency] = None
    sourceCurrency: Optional[Currency] = None
    exchangeRate: Optional[float] = None

    @field_validator("targetCurrency", "sourceCurrency", mode="before")
    def validate_currency(cls, value):
        if isinstance(value, str):
            return Currency(value)
        return value


class Account(BaseModel):
    bban: Optional[str] = None


class ReportExchangeRate(BaseModel):
    instructedAmount: Optional[InstructedAmount] = None


class NordigenTransactionModel(BaseModel):
    """
    Nordigen data transaction model.
    """

    balanceAfterTransaction: Optional[float] = None
    bankTransactionCode: Optional[str] = None
    bookingDate: Optional[datetime.date] = None
    bookingDateTime: Optional[datetime.datetime] = None
    checkId: Optional[str] = None
    creditorAccount: Optional[Account] = None
    creditorAgent: Optional[str] = None
    creditorId: Optional[str] = None
    creditorName: Optional[str] = None
    currencyExchange: Optional[Union[ReportExchangeRate, InstructedAmount]] = None
    debtorAccount: Optional[Account] = None
    debtorAgent: Optional[str] = None
    debtorName: Optional[str] = None
    endToEndId: Optional[str] = None
    entryReference: Optional[str] = None
    internalTransactionId: Optional[str] = None
    mandateId: Optional[str] = None
    merchantCategoryCode: Optional[str] = None
    proprietaryBankTransactionCode: Optional[str] = None
    purposeCode: Optional[str] = None
    remittanceInformationStructured: Optional[str] = None
    remittanceInformationStructuredArray: Optional[List[str]] = None
    remittanceInformationUnstructured: Optional[str] = None
    remmittanceInformationUnstructuredArray: Optional[List[str]] = None
    transactionAmount: Amount
    transactionId: Optional[str] = None
    ultimateCreditor: Optional[str] = None
    ultimateDebtor: Optional[str] = None
    valueDate: Optional[datetime.date] = None
    valueDateTime: Optional[datetime.datetime] = None



import os
import pytest
from ofxstatement.ui import UI

from ofxstatement_nordigen.plugin import NordigenPlugin, NordigenParser
from ofxstatement import ofx


def test_sample() -> None:
    plugin = NordigenPlugin(UI(), {})
    here = os.path.dirname(__file__)
    for filename in os.listdir(here):
        if filename.endswith(".json"):
            sample_filename = os.path.join(here, filename)
            parser = plugin.get_parser(sample_filename)
            statement = parser.parse()
            assert len(statement.lines) > 0


@pytest.mark.parametrize("filename", ["test_date.json"])
def test_parse_record(filename: str) -> None:
    here = os.path.dirname(__file__)
    sample_filename = os.path.join(here, "data", filename)
    expected_filename = sample_filename.replace(".json", ".ofx")

    parser = NordigenParser(sample_filename)
    statement = parser.parse()

    expected = open(expected_filename, "r").read()
    writer = ofx.OfxWriter(statement)
    result = writer.toxml(pretty=True)

    # Get everything between the <STMTTRNRS> and </STMTTRNRS> tags
    result = result[
        result.index("<STMTTRNRS>") : result.index("</STMTTRNRS>") + len("</STMTTRNRS>")
    ]
    expected = expected[
        expected.index("<STMTTRNRS>") : expected.index("</STMTTRNRS>")
        + len("</STMTTRNRS>")
    ]

    assert result == expected



import json
from typing import Iterable

from ofxstatement.plugin import Plugin
from ofxstatement.parser import StatementParser
from ofxstatement.statement import Statement, StatementLine

from ofxstatement_nordigen.schemas import NordigenTransactionModel


class NordigenPlugin(Plugin):
    """Retrieves Nordigen transactions and converts them to OFX format."""

    def get_parser(self, filename: str) -> "NordigenParser":
        return NordigenParser(filename)


class NordigenParser(StatementParser[str]):
    def __init__(self, filename: str) -> None:
        super().__init__()
        if not filename.endswith(".json"):
            raise ValueError("Only JSON files are supported")
        self.filename = filename

    def parse(self) -> Statement:
        """Main entry point for parsers

        super() implementation will call to split_records and parse_record to
        process the file.
        """
        with open(self.filename, "r"):
            return super().parse()

    def split_records(self) -> Iterable[str]:
        """Return iterable object consisting of a line per transaction"""
        data = json.load(open(self.filename, "r"))
        transactions = data.get("transactions", {})
        booked_transactions = transactions.get("booked", [])
        return [json.dumps(transaction) for transaction in booked_transactions]

    def parse_record(self, line: str) -> StatementLine:
        """Parse given transaction line and return StatementLine object"""

        # TODO: Infer transaction type from transaction data
        statement = StatementLine()
        transaction = json.loads(line)
        transaction_data = NordigenTransactionModel(**transaction)
        statement.id = transaction_data.transactionId
        statement.date = transaction_data.bookingDateTime
        statement.amount = transaction_data.transactionAmount.amount
        statement.memo = transaction_data.remittanceInformationUnstructured
        statement.payee = transaction_data.creditorName or transaction_data.debtorName
        statement.date_user = transaction_data.valueDateTime
        statement.check_no = transaction_data.checkId
        statement.refnum = transaction_data.internalTransactionId
        statement.currency = transaction_data.transactionAmount.currency
        if transaction_data.currencyExchange and hasattr(
            transaction_data.currencyExchange, "sourceCurrency"
        ):
            statement.orig_currency = transaction_data.currencyExchange.sourceCurrency
        return statement
